package de.unibi.airobots.bonsai.xcf;

import net.sf.xcf.util.QueueClosedException;
import net.sf.xcf.util.SynchronizedQueue;

/**
 * Creates a synchronized bound queue with a limited capacity. Before pushing an
 * object into the queue it removes elements so that the size of the queue is
 * always within the capacity.
 * 
 * @author marc
 * @author jwienke
 * @param <T>
 *            type of elements contained in this queue
 */
public abstract class BoundSynchronizedQueue<T> extends SynchronizedQueue<T> {

    private int capacity;

    /**
     * Returns the maximum capacity of this queue.
     * 
     * @return the capacity
     */
    public int getCapacity() {
        return capacity;
    }

    /**
     * Sets the maximum capacity of this queue.
     * 
     * @param capacity
     *            the capacity to set
     */
    public void setCapacity(int capacity) {
        this.capacity = capacity;
        // TODO what if this queue contains more entries than the new capacity?
    }

    /**
     * Create queue with a given capacity.
     * 
     * @param capacity
     *            the initial capacity.
     */
    public BoundSynchronizedQueue(int capacity) {
        super();
        this.capacity = capacity;
    }

    /**
     * Create queue with a capacity of 1. This implements a synchronized buffer.
     */
    public BoundSynchronizedQueue() {
        super();
        this.capacity = 1;
    }

    @Override
    public synchronized void push(T element) {
        if (super.isClosed()) {
            throw new QueueClosedException("Queue is closed!");
        }
        // discard all older elements

        while (super.getSize() >= capacity) {
            super.pop();
        }
        super.push(element);
    }

}
