package de.unibi.airobots.bonsai.xcf;

import java.util.HashSet;
import java.util.Set;

import net.sf.xcf.RemoteServer;

/**
 * Base class for managers that have a checking thread for reconnecting client
 * classes (called Reconnectables) to an infrastructure unit providing data like
 * a publisher and doing reconnection of the infrastructure unit disconnects.
 * 
 * @author jwienke
 * @param <R>
 *            managed type of
 */
public abstract class ReconnectableManager<R> {

    /**
     * Default checking interval for xcf types in ms.
     */
    public static final long DEFAULT_CHECK_INTERVAL_IN_MS = 5000;

    /**
     * Default value for {@link #isErrorOnXcfTypeInitialization()}.
     */
    public static final boolean DEFAULT_ERROR_ON_TYPE_INITIALIZATION = true;

    /**
     * Determines whether to throw an exception if the initial connection to a
     * {@link RemoteServer} fails or not and reconnect later if this succeeds.
     */
    private boolean errorOnXcfTypeInitialization =
            DEFAULT_ERROR_ON_TYPE_INITIALIZATION;

    /**
     * Initialize a xcf type with the given name. This sets up the initial
     * connection to the xcf type.
     * 
     * @param typeName
     *            name of the xcf type to initialize
     * @throws XcfTypeInitializationException
     *             thrown if the xcf type with the given name cannot be created
     *             and such errors shall be reported (
     *             {@link #isErrorOnXcfTypeInitialization()})
     */
    public void initializeXcfType(String typeName)
            throws XcfTypeInitializationException {
        Set<String> names = new HashSet<String>();
        names.add(typeName);
        initializeXcfTypes(names);
    }

    /**
     * Initializes the given list of xcf types names.
     * 
     * @param xcfTypeNames
     *            xcf types to initially connect to.
     * @throws XcfTypeInitializationException
     *             if {@link #errorOnXcfTypeInitialization} is <code>true</code>
     *             this exception will be thrown if a remote server cannot be
     *             accessed. Otherwise it will be tried to reconnect to that
     *             server from time to time.
     */
    public abstract void initializeXcfTypes(Set<String> xcfTypeNames)
            throws XcfTypeInitializationException;

    /**
     * Initializes a new R with its first xcf type and the manages it for the
     * lifetime of this class.
     * 
     * @param reconnectable
     *            reconnectable to initialize
     * @throws XcfTypeInitializationException
     *             error initializing the reconnectable
     */
    public abstract void initializeAndManage(R reconnectable)
            throws XcfTypeInitializationException;

    /**
     * Define whether to throw exceptions on initial connections to the
     * underlying xcf type or not.
     * 
     * @param error
     *            if <code>true</code>, throw exceptions, else
     *            <code>false</code>
     */
    public final void setErrorOnXcfTypeInitialization(boolean error) {
        this.errorOnXcfTypeInitialization = error;
    }

    /**
     * Tells whether to throw exceptions on initial connections to the
     * underlying xcf type or not. Default ist
     * {@value #DEFAULT_ERROR_ON_TYPE_INITIALIZATION}
     * 
     * @return <code>true</code> for throwing exceptions, else
     *         <code>false</code>
     */
    public final boolean isErrorOnXcfTypeInitialization() {
        return errorOnXcfTypeInitialization;
    }

}
