package de.unibi.airobots.bonsai.xcf.actuators;

import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

import org.apache.log4j.Logger;

import de.unibi.airobots.bonsai.actuators.SwissRangerControl;
import de.unibi.airobots.bonsai.xcf.RemoteServerReconnectable;
import de.unibi.airobots.btl.data.PoseData;

import net.sf.xcf.MethodNotFoundException;
import net.sf.xcf.RemoteServer;
import net.sf.xcf.UserException;
import net.sf.xcf.transport.XOPData;
import nu.xom.Attribute;
import nu.xom.Document;
import nu.xom.Element;
import nu.xom.ParsingException;

/**
 * This class allows you to (de-) activate the virtual laser.
 * 
 * @author jotto
 * @author jwienke
 * @author lziegler
 */
public class XcfSwissRangerControl implements RemoteServerReconnectable,
        SwissRangerControl {

    private Logger logger = Logger.getLogger(getClass());

    private String remoteServerName;
    private Object remoteServerLock = new Object();
    private RemoteServer remoteServer;

    /**
     * Constructor.
     * 
     * @param remoteServerName
     *            name of the remote server used to control the swiss ranger
     */
    public XcfSwissRangerControl(String remoteServerName) {
        this.remoteServerName = remoteServerName;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void deactivateVirtualLaser() throws IOException {

        Element elem = new Element("deactive");
        Document d = new Document(elem);

        try {
            XOPData res;
            synchronized (remoteServerLock) {
                res =
                        this.remoteServer.callMethod("deactivate", new XOPData(
                                d));
            }
            logger.debug(res.getDocument().toString());
        } catch (MethodNotFoundException e) {
            logger.debug("the method is not registered on the server", e);
            throw new IOException(e);
        } catch (UserException e) {
            throw new IOException(e);
        }

    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void activateVirtualLaser() throws IOException {

        Element elem = new Element("activate");
        Document d = new Document(elem);

        try {
            XOPData res;
            synchronized (remoteServerLock) {
                res = this.remoteServer.callMethod("activate", new XOPData(d));
            }
            logger.debug(res.getDocument());
        } catch (MethodNotFoundException e) {
            logger.debug("the method is not registered on the server", e);
            throw new IOException(e);
        } catch (UserException e) {
            throw new IOException(e);
        }

    }

    /**
     * {@inheritDoc}
     */
    @Override
    public Map<String, ErrorHandlingPolicy> getRemoteServerNames() {
        Map<String, ErrorHandlingPolicy> servers =
                new HashMap<String, ErrorHandlingPolicy>();
        servers.put(remoteServerName, ErrorHandlingPolicy.SELF_HEALING);
        return servers;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void resetRemoteServer(RemoteServer remoteServer) {
        synchronized (remoteServerLock) {
            this.remoteServer = remoteServer;
        }

    }

    @Override
    public PoseData getRawPosition(int x, int y) throws IOException {
        return getRawPosition(x, y, 1, 1);
    }

    @Override
    public PoseData getRawPosition(int x, int y, int radius) throws IOException {
        return getRawPosition(x, y, radius, 1);
    }

    @Override
    public PoseData getRawPosition(int x, int y, int radius, int frames)
            throws IOException {
        
        if (radius < 1 || frames < 1) {
            throw new IOException("radius or frames not in range!");
        }
        
        Element elem = new Element("getRawPosition");
        elem.addAttribute(new Attribute("x", Integer.toString(x)));
        elem.addAttribute(new Attribute("y", Integer.toString(y)));
        elem.addAttribute(new Attribute("radius", Integer.toString(radius)));
        elem.addAttribute(new Attribute("frames", Integer.toString(frames)));
        Document d = new Document(elem);
        PoseData pose;

        try {
            XOPData res;
            synchronized (remoteServerLock) {
                res =
                        this.remoteServer.callMethod("getRawPosition",
                                new XOPData(d));
            }
            logger.debug(res.getDocument().toString());
            pose = PoseData.fromDocument(res.getDocument(), PoseData.class);
        } catch (MethodNotFoundException e) {
            logger.debug("the method is not registered on the server", e);
            throw new IOException(e);
        } catch (UserException e) {
            throw new IOException(e);
        } catch (ParsingException e) {
            throw new IOException(e);
        }
        return pose;
    }

}
