package de.unibi.airobots.bonsai.xcf.configuration;

import java.util.Map;

import de.unibi.airobots.bonsai.core.Sensor;
import de.unibi.airobots.bonsai.core.CoreObjectFactory.SensorToConfigure;
import de.unibi.airobots.bonsai.xcf.sensors.XcfFaceIdentificationSensor;
import de.unibi.airobots.btl.data.FaceIdentificationList;

/**
 * Configuration for {@link XcfFaceIdentificationSensor}.
 * <p>
 * Available options:
 * <ul>
 * <li>{@value #OPTION_RESULT_PUBLISHER_NAME} (required): name of the publisher
 * returning the classification results</li>
 * <li>{@value #OPTION_ACTIVATION_SERVER_NAME} (required): name of the the
 * server used to activate and deactivate the classification chain</li>
 * <li>{@value #OPTION_CONTROL_SERVER_NAME} (required): name of the server used
 * to control classification and learning</li>
 * </ul>
 * 
 * @author jwienke
 */
public class ConfiguredXcfFaceIdentificationSensor extends
        ConfiguredXcfSensor<FaceIdentificationList> {

    /**
     * Name of the options defining the publisher that returns classification
     * results.
     */
    public static final String OPTION_RESULT_PUBLISHER_NAME = "resultPublisher";

    /**
     * Name of the option defining the server to activate classification.
     */
    public static final String OPTION_ACTIVATION_SERVER_NAME =
            "activationServer";

    /**
     * Name of the option defining the server to control classification and
     * learning.
     */
    public static final String OPTION_CONTROL_SERVER_NAME = "controlServer";

    private String resultSubscriberName = "";
    private String activationServerName = "";
    private String controlServerName = "";

    /**
     * {@inheritDoc}
     */
    @Override
    public Sensor<FaceIdentificationList> createInstance() {
        return new XcfFaceIdentificationSensor(resultSubscriberName,
                activationServerName, controlServerName);
    }

    /**
     * Checks if the sensor that can be produced is suitable for the requested
     * configuration.
     * 
     * @param sensor
     *            requested configuration
     * @return <code>true</code> if suitable
     */
    public static boolean isSuitableFor(SensorToConfigure sensor) {
        return sensor.getSensorClass()
                .equals(XcfFaceIdentificationSensor.class)
                && FaceIdentificationList.class.isAssignableFrom(sensor
                        .getDataTypeClass());
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public String[] getSubscribersToInitialize() {
        return new String[] { resultSubscriberName };
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void parseOptions(Map<String, String> options)
            throws IllegalArgumentException {

        if (!options.containsKey(OPTION_ACTIVATION_SERVER_NAME)) {
            throw new IllegalArgumentException("Option '"
                    + OPTION_ACTIVATION_SERVER_NAME + "' missing.");
        }
        if (!options.containsKey(OPTION_CONTROL_SERVER_NAME)) {
            throw new IllegalArgumentException("Option '"
                    + OPTION_CONTROL_SERVER_NAME + "' missing.");
        }
        if (!options.containsKey(OPTION_RESULT_PUBLISHER_NAME)) {
            throw new IllegalArgumentException("Option '"
                    + OPTION_RESULT_PUBLISHER_NAME + "' missing.");
        }

        resultSubscriberName = options.get(OPTION_RESULT_PUBLISHER_NAME);
        activationServerName = options.get(OPTION_ACTIVATION_SERVER_NAME);
        controlServerName = options.get(OPTION_CONTROL_SERVER_NAME);

    }

    /**
     * {@inheritDoc}
     */
    @Override
    public String[] getRemoteServersToInitialize() {
        return new String[] { activationServerName, controlServerName };
    }

}
