package de.unibi.airobots.bonsai.xcf.sensors;

import java.io.IOException;
import java.util.LinkedList;
import java.util.List;

import org.apache.log4j.Logger;

import net.sf.xcf.ActiveMemory;
import net.sf.xcf.event.MemoryEvent;
import net.sf.xcf.event.MemoryEventAdapter;
import net.sf.xcf.event.MemoryEventListener;
import net.sf.xcf.memory.MemoryAction;
import net.sf.xcf.memory.MemoryException;
import net.sf.xcf.transport.XOPData;
import net.sf.xcf.xml.XPath;
import nu.xom.Attribute;
import nu.xom.Element;
import nu.xom.ParsingException;
import de.unibi.airobots.bonsai.core.SensorListener;
import de.unibi.airobots.bonsai.sensors.RobotStateSensor;
import de.unibi.airobots.bonsai.xcf.ActiveMemoryReconnectable;
import de.unibi.airobots.btl.data.RobotState;

/**
 * @author ltwardon
 */
public class XcfRobotStateSensor implements ActiveMemoryReconnectable, RobotStateSensor{
    
    private Logger logger = Logger.getLogger(getClass());
    
    private String memoryName = "";
    private Object memoryLock = new Object();
    private Object sharedDataLock = new Object();
    private ActiveMemory memory = null;
    
    /**
     * Namespace of active memory xml documents.
     */
    public static final String NAMESPACE_DBXML =
        "http://www.sleepycat.com/2002/dbxml";
    
    private List<SensorListener<RobotState>> listeners =
        new LinkedList<SensorListener<RobotState>>();
    
    private RobotState robotState;

    /**
     * Own listener used to intercept the memory for person changes.
     */
    private MemoryEventListener selfListener;
    
    /**
     * Constructor.
     * 
     * @param memoryName
     *            name of the active memory to get the persons from
     */
    public XcfRobotStateSensor(String memoryName) {

        this.memoryName = memoryName;

        logger.debug("Create memory sensor for memoryName " + memoryName);

        selfListener =
                new MemoryEventAdapter(MemoryAction.ALL, new XPath(
                        RobotState.getBaseTag())) {

                    @Override
                    public void handleEvent(MemoryEvent e) {
                        handleMemoryEvent(e);
                    }
                };
    }

    private void handleMemoryEvent(MemoryEvent e) {
        try {

            synchronized (sharedDataLock) {

                // create BTL type
                XOPData data = e.getData();
                Element root = data.getDocument().getRootElement();
                if (root == null) {
                    logger.error("Received Document has no root node");
                    return;
                }
                RobotState robotState = new RobotState();
                RobotState.fromElement(root, robotState);

                // extract xcf id
                Attribute att = root.getAttribute("id", NAMESPACE_DBXML);
                if (att == null) {
                    logger.error("Received Document has no xcf id");
                    return;
                }
                int id = Integer.parseInt(att.getValue());
                
                synchronized (sharedDataLock) {
                
                    MemoryAction a = e.getType();
                    if (a == MemoryAction.INSERT) {
                        logger.debug("insert id " + id);
                        this.robotState = robotState;
                    } else if (e.getType() == MemoryAction.REPLACE) {
                        logger.debug("replace id " + id);
                        this.robotState = robotState;
                    } else if (e.getType() == MemoryAction.REMOVE) {
                        logger.debug("remove id " + id);
                        this.robotState = null;
                    }
    
                    for (SensorListener<RobotState> l : listeners) {
                        l.newDataAvailable(this.robotState);
                    }
                
                }

            }

        } catch (ParsingException ex) {
            logger.error(ex.getMessage());
            logger.debug(ex);
        } catch (NumberFormatException ex) {
            logger.error("Received Document has not a valid xcf id");
            logger.debug(ex);
        }
    }
    
    @Override
    public Class<RobotState> getDataType() {
        return RobotState.class;
    }

    @Override
    public RobotState readLast() throws IOException, InterruptedException {
        return robotState;
    }

    @Override
    public RobotState readLast(long timeout) throws IOException,
            InterruptedException {
        return robotState;
    }

    @Override
    public boolean hasNext() {
        return robotState!=null;
    }

    @Override
    public void addSensorListener(SensorListener<RobotState> listener) {
        listeners.add(listener);
    }

    @Override
    public void removeSensorListener(SensorListener<RobotState> listener) {
        listeners.remove(listener);
    }

    @Override
    public String getActiveMemoryName() {
        return memoryName;
    }

    @Override
    public void resetActiveMemory(ActiveMemory activeMemory) {
        
        synchronized (memoryLock) {
            this.memory = activeMemory;

            try {
                memory.addListener(selfListener);
            } catch (MemoryException e) {

            }
        }
        
    }

}
