package de.unibi.airobots.bonsai.core;

import static org.junit.Assert.*;

import java.io.IOException;
import java.net.URI;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import org.apache.log4j.BasicConfigurator;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

import de.unibi.airobots.bonsai.core.BonsaiManager.ConfigurationException;
import de.unibi.airobots.bonsai.core.configuration.ConfigurationParser;
import de.unibi.airobots.bonsai.core.configuration.ConfiguredActuator;
import de.unibi.airobots.bonsai.core.configuration.ConfiguredCoreObjectFactory;
import de.unibi.airobots.bonsai.core.configuration.ConfiguredSensor;

/**
 * Test cases for the {@link BonsaiManager}.
 * 
 * @author jwienke
 */
public class BonsaiManagerTest {

    /**
     * A simple dummy configurator for the unit test.
     * 
     * @author jwienke
     */
    private class DummyConfigurator implements ConfigurationParser {

        private boolean illegalState = false;
        private boolean ioException = false;
        private boolean parserError = false;
        private boolean illegalParseState = false;

        private Set<ConfiguredActuator> configuredActuators =
                new HashSet<ConfiguredActuator>();
        private Set<ConfiguredCoreObjectFactory> configuredFactories =
                new HashSet<ConfiguredCoreObjectFactory>();
        private Set<ConfiguredSensor> configuredSensors =
                new HashSet<ConfiguredSensor>();

        @Override
        public Set<ConfiguredActuator> getConfiguredActuators()
                throws IllegalStateException {
            if (illegalState) {
                throw new IllegalStateException();
            }
            return configuredActuators;
        }

        @Override
        public Set<ConfiguredCoreObjectFactory> getConfiguredFactories()
                throws IllegalStateException {
            if (illegalState) {
                throw new IllegalStateException();
            }
            return configuredFactories;
        }

        @Override
        public Set<ConfiguredSensor> getConfiguredSensors()
                throws IllegalStateException {
            if (illegalState) {
                throw new IllegalStateException();
            }
            return configuredSensors;
        }

        @Override
        public void parseFile(URI configurationUrl) throws IOException,
                ParseException, IllegalStateException {
            if (ioException) {
                throw new IOException();
            }
            if (parserError) {
                throw new ParseException("");
            }
            if (illegalParseState) {
                throw new IllegalStateException();
            }
        }

    }

    private DummyConfigurator configurator;

    /**
     * Setup logging.
     * 
     * @throws Exception
     *             error
     */
    @BeforeClass
    public static void setUpBeforeClass() throws Exception {
        BasicConfigurator.configure();
    }

    /**
     * Setup method constructing a new "singleton" instance of
     * {@link BonsaiManager}.
     */
    @Before
    public void setUp() {
        BonsaiManager.testConstructNewInstance();
        configurator = new DummyConfigurator();
        BonsaiManager.getInstance().setParser(configurator);
    }

    @Test(expected = ConfigurationException.class)
    public void parseErrorIOException() throws Exception {
        configurator.ioException = true;
        BonsaiManager.getInstance().configure("dummy");
    }

    @Test(expected = ConfigurationException.class)
    public void parseErrorParseException() throws Exception {
        configurator.parserError = true;
        BonsaiManager.getInstance().configure("dummy");
    }

    @Test(expected = IllegalStateException.class)
    public void illegealState() throws Exception {
        configurator.illegalParseState = true;
        BonsaiManager.getInstance().configure("dummy");
    }

    @Test(expected = ConfigurationException.class)
    public void unknownFactoryClass() throws Exception {

        ConfiguredCoreObjectFactory f = new ConfiguredCoreObjectFactory();
        f.setFactoryClassName("I.dont.know.You");
        f.setFactroyOptions(new HashMap<String, String>());
        configurator.configuredFactories.add(f);

        BonsaiManager.getInstance().configure("dummy");

    }

    @Test(expected = ConfigurationException.class)
    public void noFactoryClass() throws Exception {

        ConfiguredCoreObjectFactory f = new ConfiguredCoreObjectFactory();
        f.setFactoryClassName(String.class.getName());
        f.setFactroyOptions(new HashMap<String, String>());
        configurator.configuredFactories.add(f);

        BonsaiManager.getInstance().configure("dummy");

    }

    @Test(expected = ConfigurationException.class)
    public void duplicateFactoryClass() throws Exception {

        ConfiguredCoreObjectFactory f = new ConfiguredCoreObjectFactory();
        f.setFactoryClassName(String.class.getName());
        f.setFactroyOptions(new HashMap<String, String>());
        configurator.configuredFactories.add(f);
        ConfiguredCoreObjectFactory f2 = new ConfiguredCoreObjectFactory();
        f2.setFactoryClassName(String.class.getName());
        f2.setFactroyOptions(new HashMap<String, String>());
        configurator.configuredFactories.add(f2);

        BonsaiManager.getInstance().configure("dummy");

    }

    @Test
    public void workingFactory() throws Exception {

        ConfiguredCoreObjectFactory f = new ConfiguredCoreObjectFactory();
        f.setFactoryClassName(TestFactory.class.getName());
        f.setFactroyOptions(new HashMap<String, String>());
        configurator.configuredFactories.add(f);

        BonsaiManager.getInstance().configure("dummy");

        assertEquals(1, BonsaiManager.getInstance()
                .getConfiguredFactoryClasses().size());
        assertTrue(BonsaiManager.getInstance().getConfiguredFactoryClasses()
                .contains(TestFactory.class));

    }

    @Test(expected = ConfigurationException.class)
    public void factoryConfigurationError() throws Exception {

        ConfiguredCoreObjectFactory f = new ConfiguredCoreObjectFactory();
        f.setFactoryClassName(TestFactory.class.getName());
        Map<String, String> options = new HashMap<String, String>();
        options.put(TestFactory.FAIL_KEY, "i will fail misserably");
        f.setFactroyOptions(options);
        configurator.configuredFactories.add(f);

        BonsaiManager.getInstance().configure("dummy");

    }

    @Test(expected = ConfigurationException.class)
    public void duplicateKeyError() throws Exception {

        ConfiguredCoreObjectFactory f = new ConfiguredCoreObjectFactory();
        f.setFactoryClassName(TestFactory.class.getName());
        Map<String, String> options = new HashMap<String, String>();
        f.setFactroyOptions(options);
        configurator.configuredFactories.add(f);

        ConfiguredSensor s1 = new ConfiguredSensor();
        s1.setKey("duplicate");
        s1.setDataType(String.class.getName());
        s1.setFactoryClassName(TestFactory.class.getName());
        s1.setSensorClassName(TestSensor.class.getName());
        s1.setOptions(new HashMap<String, String>());
        configurator.configuredSensors.add(s1);

        ConfiguredSensor s2 = new ConfiguredSensor();
        s2.setKey(s1.getKey());
        s2.setDataType(String.class.getName());
        s2.setFactoryClassName(TestFactory.class.getName());
        s2.setSensorClassName(TestSensor.class.getName());
        s2.setOptions(new HashMap<String, String>());
        configurator.configuredSensors.add(s2);

        BonsaiManager.getInstance().configure("dummy");

    }

    @Test
    public void correctSensorCreation() throws Exception {

        final String key = "testKey";

        ConfiguredCoreObjectFactory f = new ConfiguredCoreObjectFactory();
        f.setFactoryClassName(TestFactory.class.getName());
        Map<String, String> options = new HashMap<String, String>();
        f.setFactroyOptions(options);
        configurator.configuredFactories.add(f);

        ConfiguredSensor s1 = new ConfiguredSensor();
        s1.setKey(key);
        s1.setDataType(String.class.getName());
        s1.setSensorClassName(TestSensor.class.getName());
        s1.setFactoryClassName(TestFactory.class.getName());
        s1.setOptions(new HashMap<String, String>());
        configurator.configuredSensors.add(s1);

        BonsaiManager.getInstance().configure("dummy");

        assertTrue(BonsaiManager.getInstance().canCreateSensor(key,
                String.class));
        assertFalse(BonsaiManager.getInstance().canCreateSensor(key,
                Integer.class));
        assertFalse(BonsaiManager.getInstance().canCreateSensor("dummyblaa",
                String.class));

        assertNotNull(BonsaiManager.getInstance().createSensor(key,
                String.class));

        assertTrue(BonsaiManager.getInstance().createSensor(key, String.class) instanceof TestSensor<?>);

        // ensure that creating an unknown key-type combination fails
        try {
            BonsaiManager.getInstance().createSensor(key, Integer.class);
            fail("Must throw an exception "
                    + "for an unknown key-type combination.");
        } catch (IllegalArgumentException e) {
        }

        try {
            BonsaiManager.getInstance().createSensor(key + "1", String.class);
            fail("Must throw an exception "
                    + "for an unknown key-type combination.");
        } catch (IllegalArgumentException e) {
        }

    }

    @Test
    public void correctActuatorCreation() throws Exception {

        final String key = "testKey";

        ConfiguredCoreObjectFactory f = new ConfiguredCoreObjectFactory();
        f.setFactoryClassName(TestFactory.class.getName());
        Map<String, String> options = new HashMap<String, String>();
        f.setFactroyOptions(options);
        configurator.configuredFactories.add(f);

        ConfiguredActuator actuator = new ConfiguredActuator();
        actuator.setKey(key);
        actuator.setActuatorClassName(FooActuatorImpl.class.getName());
        actuator.setInterfaceClassName(FooActuator.class.getName());
        actuator.setFactoryClassName(TestFactory.class.getName());
        actuator.setOptions(new HashMap<String, String>());
        configurator.configuredActuators.add(actuator);

        BonsaiManager.getInstance().configure("dummy");

        // check that derived instantiation is possible
        assertTrue(BonsaiManager.getInstance().canCreateActuator(key,
                FooActuator.class));
        assertTrue(BonsaiManager.getInstance().canCreateActuator(key,
                FooActuatorImpl.class));
        assertFalse(BonsaiManager.getInstance().canCreateSensor(key,
                DerivedFooActuatorImpl.class));

        // wrong key should not be possible
        assertFalse(BonsaiManager.getInstance().canCreateActuator(key + "bla",
                FooActuator.class));
        assertFalse(BonsaiManager.getInstance().canCreateActuator(key + "bla",
                FooActuatorImpl.class));
        assertFalse(BonsaiManager.getInstance().canCreateSensor(key + "bla",
                DerivedFooActuatorImpl.class));

        assertNotNull(BonsaiManager.getInstance().createActuator(key,
                FooActuator.class));
        assertNotNull(BonsaiManager.getInstance().createActuator(key,
                FooActuatorImpl.class));

        assertTrue(BonsaiManager.getInstance().createActuator(key,
                FooActuator.class) instanceof FooActuatorImpl);
        assertTrue(BonsaiManager.getInstance().createActuator(key,
                FooActuatorImpl.class) instanceof FooActuatorImpl);

        // ensure that creating an unknown key-type combination etc fails
        try {
            BonsaiManager.getInstance().createActuator(key,
                    UnknownActuator.class);
            fail("Must throw an exception for an unknown key-type combination.");
        } catch (IllegalArgumentException e) {
        }

        try {
            BonsaiManager.getInstance().createActuator(key + "test",
                    FooActuatorImpl.class);
            fail("Must throw an exception for an unknown key-type combination.");
        } catch (IllegalArgumentException e) {
        }

    }

}
