package de.unibi.airobots.bonsai.xcf;

import static org.easymock.classextension.EasyMock.*;
import static org.junit.Assert.*;

import java.util.Timer;
import java.util.TimerTask;

import net.sf.xcf.RemoteServer;
import net.sf.xcf.event.system.SystemEventAdapter;
import net.sf.xcf.event.system.SystemEventListener;
import net.sf.xcf.transport.XOPData;

import org.apache.log4j.BasicConfigurator;
import org.easymock.Capture;
import org.junit.BeforeClass;
import org.junit.Test;

import Ice.ConnectFailedException;

/**
 * Test cases for {@link ConnectionErrorResistantRemoteServer}.
 * 
 * @author jwienke
 */
public class ConnectionErrorResistantRemoteServerTest {

    /**
     * Static initializer for logging.
     */
    @BeforeClass
    public static void setUpLogging() {
        BasicConfigurator.configure();
    }

    /**
     * Tests the general behaviour for an error condition.
     * 
     * @throws Exception
     *             error
     */
    @Test
    public void behaviourOnError() throws Exception {

        final String serverName = "testServer";

        final RemoteServer failingServerMock = createMock(RemoteServer.class);
        expect(failingServerMock.getServerName()).andReturn(serverName)
                .anyTimes();
        final RemoteServer resetServerMock = createMock(RemoteServer.class);
        expect(resetServerMock.getServerName()).andReturn(serverName)
                .anyTimes();

        RemoteServerReconnectableManager managerMock =
                createMock(RemoteServerReconnectableManager.class);

        final ConnectionErrorResistantRemoteServer decorator =
                new ConnectionErrorResistantRemoteServer(managerMock);
        decorator.resetRemoteServer(failingServerMock);

        XOPData data = XOPData.fromString("<test/>");
        final String methodName = "testMethod";
        expect(failingServerMock.callMethod(methodName, data)).andThrow(
                new ConnectFailedException());

        // this capture is used to start a timer that initiates the asynchronous
        // resetting of the remote server
        Capture<String> serverRecreator = new Capture<String>() {

            /**
             * Serial Version UID
             */
            private static final long serialVersionUID = -1951691258307880524L;

            private String value = null;

            /**
             * {@inheritDoc}
             */
            @Override
            public void setValue(String value) {
                this.value = value;
                new Timer().schedule(new TimerTask() {

                    /**
                     * {@inheritDoc}
                     */
                    @Override
                    public void run() {
                        decorator.resetRemoteServer(resetServerMock);
                    }

                }, 100);
            }

            /**
             * {@inheritDoc}
             */
            @Override
            public boolean hasCaptured() {
                return value != null;
            }

        };
        managerMock.markRemoteServerAsDisconnected(and(eq(serverName),
                capture(serverRecreator)));

        expect(resetServerMock.callMethod(methodName, data)).andReturn(
                XOPData.fromString("<foo/>"));

        replay(failingServerMock, resetServerMock, managerMock);

        decorator.callMethod(methodName, data);

        Thread.sleep(300);

        verify(failingServerMock, resetServerMock, managerMock);

    }

    /**
     * Tests waiting for the first server in a method call on the decorator.
     * 
     * @throws Exception
     *             error
     */
    @Test
    public void lazyServerInit() throws Exception {

        final String serverName = "testServer";
        final String methodName = "testMethod";
        final XOPData data = XOPData.fromString("<test/>");
        final XOPData expectedResult = XOPData.fromString("<result/>");

        final RemoteServer remoteServerMock = createMock(RemoteServer.class);
        expect(remoteServerMock.getServerName()).andReturn(serverName)
                .anyTimes();
        expect(remoteServerMock.callMethod(methodName, data)).andReturn(
                expectedResult);

        RemoteServerReconnectableManager managerMock =
                createMock(RemoteServerReconnectableManager.class);

        final ConnectionErrorResistantRemoteServer decorator =
                new ConnectionErrorResistantRemoteServer(managerMock);

        new Timer().schedule(new TimerTask() {

            @Override
            public void run() {
                decorator.resetRemoteServer(remoteServerMock);
            }
        }, 150);

        replay(remoteServerMock, managerMock);

        XOPData result = decorator.callMethod(methodName, data);
        assertEquals(expectedResult, result);

        verify(remoteServerMock, managerMock);

    }

    /**
     * Ensure that listeners are re-assigned to the new wrapped server instance.
     * 
     * @throws Exception
     *             error
     */
    @Test
    public void listenerReassignment() throws Exception {

        final String serverName = "fgaksdfjaklsdf";

        RemoteServer failingRemoteServerMock = createMock(RemoteServer.class);
        expect(failingRemoteServerMock.getServerName()).andStubReturn(
                serverName);
        RemoteServer secondRemoteServerMock = createMock(RemoteServer.class);
        expect(secondRemoteServerMock.getServerName())
                .andStubReturn(serverName);

        RemoteServerReconnectableManager managerMock =
                createMock(RemoteServerReconnectableManager.class);
        ConnectionErrorResistantRemoteServer wrapper =
                new ConnectionErrorResistantRemoteServer(managerMock);

        SystemEventListener l1 = new SystemEventAdapter() {
        };
        SystemEventListener l2 = new SystemEventAdapter() {
        };
        SystemEventListener l3 = new SystemEventAdapter() {
        };

        failingRemoteServerMock.addListener(l1);
        failingRemoteServerMock.addListener(l2);
        failingRemoteServerMock.addListener(l3);
        failingRemoteServerMock.removeListener(l3);

        secondRemoteServerMock.addListener(l1);
        secondRemoteServerMock.addListener(l2);

        replay(failingRemoteServerMock, secondRemoteServerMock, managerMock);

        wrapper.resetRemoteServer(failingRemoteServerMock);

        wrapper.addListener(l1);
        wrapper.addListener(l2);
        wrapper.addListener(l3);

        wrapper.removeListener(l3);

        wrapper.resetRemoteServer(secondRemoteServerMock);

        verify(failingRemoteServerMock, secondRemoteServerMock, managerMock);

    }

}
