/**
 * 
 */
package de.unibi.airobots.bonsai.xcf;

import static org.easymock.classextension.EasyMock.*;

import net.sf.xcf.InitializeException;
import net.sf.xcf.Subscriber;
import net.sf.xcf.XcfManager;
import net.sf.xcf.naming.NameNotFoundException;

import org.apache.log4j.BasicConfigurator;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

/**
 * Test cases for {@link #SubscriberReconnectableManagerTest()}.
 * 
 * @author jwienke
 */
public class SubscriberReconnectableManagerTest {

    private XcfManager xcfManagerMock;
    private SubscriberReconnectableManager manager;

    /**
     * Static initializer for logging.
     */
    @BeforeClass
    public static void setUpLogging() {
        BasicConfigurator.configure();
    }

    /**
     * Set up fixture.
     * 
     * @throws Exception
     *             error...
     */
    @Before
    public void setUp() throws Exception {
        xcfManagerMock = createMock(XcfManager.class);
        manager = new SubscriberReconnectableManager(xcfManagerMock);
    }

    /**
     * Ensure that a subscriber for the desired name is requested at the
     * {@link XcfManager}.
     * 
     * @throws Exception
     *             error
     */
    @Test
    public void initializeSubscriberCorrect() throws Exception {

        final String subscriberName = "nameMe";

        Subscriber subscriberMock = createMock(Subscriber.class);

        expect(xcfManagerMock.createSubscriber(subscriberName)).andReturn(
                subscriberMock);

        replay(xcfManagerMock);

        manager.initializeXcfType(subscriberName);

        verify(xcfManagerMock);

    }

    /**
     * Report error on subscriber connection if desired.
     * 
     * @throws Exception
     *             desired error
     */
    @Test(expected = XcfTypeInitializationException.class)
    public void initializeSubscriberWrongNameError() throws Exception {

        manager.setErrorOnXcfTypeInitialization(true);

        final String subscriberName = "nameMe";

        expect(xcfManagerMock.createSubscriber(subscriberName)).andThrow(
                new NameNotFoundException());

        replay(xcfManagerMock);

        manager.initializeXcfType(subscriberName);

        verify(xcfManagerMock);

    }

    /**
     * Ignore errors on subscriber connection if desired.
     * 
     * @throws Exception
     *             error
     */
    @Test
    public void initializeSubscriberWrongNameNoError() throws Exception {

        manager.setErrorOnXcfTypeInitialization(false);

        final String subscriberName = "nameMe";

        expect(xcfManagerMock.createSubscriber(subscriberName)).andThrow(
                new NameNotFoundException());

        replay(xcfManagerMock);

        manager.initializeXcfType(subscriberName);

        verify(xcfManagerMock);

    }

    /**
     * Test that the checking thread is executed.
     * 
     * @throws Exception
     *             error
     */
    @Test
    public void checkingWorks() throws Exception {

        final String subscriberName = "nameMe";

        Subscriber subscriberMock = createMock(Subscriber.class);

        expect(xcfManagerMock.createSubscriber(subscriberName)).andReturn(
                subscriberMock).times(2, 100);

        replay(xcfManagerMock, subscriberMock);

        manager = new SubscriberReconnectableManager(xcfManagerMock, 200);
        manager.initializeXcfType(subscriberName);

        Thread.sleep(500);

        verify(xcfManagerMock, subscriberMock);

    }

    /**
     * Ensure that the subscriber is set in every
     * {@link SubscriberReconnectable} after connection.
     * 
     * @throws Exception
     *             error
     */
    @Test
    public void initiallySettingSubscriber() throws Exception {

        final String subName1 = "name1";
        final String subName2 = "barfoo";

        // create mock subscribers to use
        Subscriber subscriberMock1 = createMock(Subscriber.class);
        Subscriber subscriberMock2 = createMock(Subscriber.class);

        // xcf manager mock must return these mocks
        expect(xcfManagerMock.createSubscriber(subName1)).andReturn(
                subscriberMock1);
        expect(xcfManagerMock.createSubscriber(subName2)).andReturn(
                subscriberMock2);

        // we want explicit errors because there shall be no errors now
        manager.setErrorOnXcfTypeInitialization(true);

        // create mock objects for the reconnectables
        SubscriberReconnectable reconnectable1 =
                createMock(SubscriberReconnectable.class);
        expect(reconnectable1.getSubscriberName()).andReturn(subName1)
                .anyTimes();
        SubscriberReconnectable reconnectable2 =
                createMock(SubscriberReconnectable.class);
        expect(reconnectable2.getSubscriberName()).andReturn(subName2)
                .anyTimes();

        // let the mock reconnectables expect a resetSubscriber call for
        // initialization
        reconnectable1.resetSubscriber(subscriberMock1);
        reconnectable2.resetSubscriber(subscriberMock2);

        replay(subscriberMock1, subscriberMock2, xcfManagerMock,
                reconnectable1, reconnectable2);

        // initialize subscriber reconnectable manager with the desired
        // subscribers
        manager.initializeXcfType(subName1);
        manager.initializeXcfType(subName2);

        // manage 2 reconnectables
        manager.initializeAndManage(reconnectable1);
        manager.initializeAndManage(reconnectable2);

        verify(subscriberMock1, subscriberMock2, xcfManagerMock,
                reconnectable1, reconnectable2);

    }

    /**
     * Ensure that the subscriber is set if connection error shall not be
     * reported and the connection to the subscriber succeeds afterwards.
     * 
     * @throws Exception
     *             error
     */
    @Test
    public void settingSubscriberAfterInitialConnectionError() throws Exception {

        final String subscriberName = "testName";

        Subscriber subscriberMock = createMock(Subscriber.class);

        // first creating the subscriber shall fail, later it shall work
        expect(xcfManagerMock.createSubscriber(subscriberName)).andThrow(
                new NameNotFoundException());
        expect(xcfManagerMock.createSubscriber(subscriberName)).andThrow(
                new InitializeException());
        expect(xcfManagerMock.createSubscriber(subscriberName)).andReturn(
                subscriberMock).atLeastOnce();

        // the reconnectable mock which ensures that there was a subscriber set
        SubscriberReconnectable reconnectableMock =
                createMock(SubscriberReconnectable.class);
        expect(reconnectableMock.getSubscriberName()).andReturn(subscriberName)
                .anyTimes();
        reconnectableMock.resetSubscriber(subscriberMock);

        replay(xcfManagerMock, subscriberMock, reconnectableMock);

        final long interval = 100;
        manager = new SubscriberReconnectableManager(xcfManagerMock, interval);
        manager.setErrorOnXcfTypeInitialization(false);

        manager.initializeXcfType(subscriberName);
        manager.initializeAndManage(reconnectableMock);

        Thread.sleep(4 * interval);

        verify(xcfManagerMock, subscriberMock, reconnectableMock);

    }

    /**
     * Ensure that the {@link Subscriber} is reset after a connection error in
     * between work.
     * 
     * @throws Exception
     *             error
     */
    @Test
    public void resetSubscriberAfterError() throws Exception {

        Subscriber subscriberBeforeErrorMock = createMock(Subscriber.class);
        Subscriber subscriberAfterErrorMock = createMock(Subscriber.class);

        final String subscriberName = "bfdasrfsf";

        // first successfully create one subscriber, afterwards issue an error
        // and the create a new subscriber
        expect(xcfManagerMock.createSubscriber(subscriberName)).andReturn(
                subscriberBeforeErrorMock).times(3);
        expect(xcfManagerMock.createSubscriber(subscriberName)).andThrow(
                new NameNotFoundException());
        expect(xcfManagerMock.createSubscriber(subscriberName)).andReturn(
                subscriberAfterErrorMock).atLeastOnce();
        makeThreadSafe(xcfManagerMock, true);

        // both subscribers must be set in the managed reconnectable
        SubscriberReconnectable reconnectableMock =
                createMock(SubscriberReconnectable.class);
        expect(reconnectableMock.getSubscriberName()).andReturn(subscriberName)
                .anyTimes();
        reconnectableMock.resetSubscriber(subscriberBeforeErrorMock);
        reconnectableMock.resetSubscriber(subscriberAfterErrorMock);

        replay(xcfManagerMock, subscriberAfterErrorMock,
                subscriberBeforeErrorMock, reconnectableMock);

        final long interval = 100;
        manager = new SubscriberReconnectableManager(xcfManagerMock, interval);
        manager.initializeXcfType(subscriberName);
        manager.initializeAndManage(reconnectableMock);

        Thread.sleep(7 * interval);

        verify(xcfManagerMock, subscriberAfterErrorMock,
                subscriberBeforeErrorMock, reconnectableMock);

    }

}
